import path from "node:path";
import fs from "node:fs";
import util from "node:util";
import child_process from "node:child_process";

const PLUGIN_NAME = "examplePlugin";
const BASKET_PATH = "../../wbk/dist";
const DIST_PATH = path.join(BASKET_PATH, "plugins", PLUGIN_NAME);
const PRODUCTION_BUILD = process.argv.includes("-production");
const WATCH = process.argv.includes("-watch");

try {
    console.log("Cleaning distribution directory...");
    await fs.promises.rm(DIST_PATH, { recursive: true, force: true });

    console.log("Copying files...");
    await fs.promises.mkdir(DIST_PATH, { recursive: true });
    await fs.promises.copyFile("src/plugin.json", path.join(DIST_PATH, "plugin.json"));
    await fs.promises.copyFile("CHANGELOG.md", path.join(DIST_PATH, "CHANGELOG.md"));
    await fs.promises.copyFile("README.md", path.join(DIST_PATH, "README.md"));

    console.log("Updating plugin version...");
    await replaceVersionInFile(path.join(DIST_PATH, "plugin.json"));

    console.log("Registering plugin...");
    await spawnChildProcess("updatePlugins", "node", ["updatePlugins.js"], BASKET_PATH + "/plugins");

    console.log("Running webpack...");
    await spawnChildProcess("Webpack", "node", [
        "node_modules/webpack-cli/bin/cli",
        WATCH ? "watch" : "build",
        "-c", "webpack.config",
        "--node-env", PRODUCTION_BUILD ? "production" : "development",
        "--env", `distPath=${path.resolve(DIST_PATH)}`,
        "--progress"
    ]);

    console.log("Build finished.");
} catch (pError) {
    console.error("\nBuild failed: " + pError);
    process.exitCode = 1;
}

/************ 
 * utilities 
 ************/

async function replaceVersionInFile(pFile) {
    const readFile = util.promisify(fs.readFile);
    const writeFile = util.promisify(fs.writeFile);
    
    const tBuffer = await readFile(pFile, { encoding: "utf8" });
    let tFileContent = tBuffer.toString();
    tFileContent = tFileContent.replaceAll("@@PLUGIN_VERSION", process.env.npm_package_version);
    await writeFile(pFile, tFileContent);
}

function spawnChildProcess(pName, pCmd, pArgs, pCwd) {
    const tPromise = new Promise((resolve, reject) => {
        const tProcess = child_process.spawn(pCmd, pArgs, {
            stdio: "inherit",
            cwd: pCwd,
            shell: true
        });
        tProcess.on("close", (pCode) => {
            if (pCode === 0) {
                resolve();
            } else {
                reject(`${pName} failed!`);
            }
        });
    });
    return (tPromise);
}