import { HostMessageType, MessageApiVersion, BaseMessage } from "./MessageApi";
import { GeneratedReport, SessionModifications } from "./HostParameters";

/** Base interface for host messages. */
export interface HostMessage extends BaseMessage
{
    type: HostMessageType;
}

/**
 * Will be send during initialization of the basket to request the configuration.
 * A `wbk.configuration` message has to be send back to the basket.
 */
export interface HostGetConfigurationMessage extends HostMessage
{
    type: "wbkHost.getConfiguration";
    parameter: {
        appVersion: string;
        messageApiVersion: MessageApiVersion;
    };
}

/**
 * Will be send to request the full article data of a custom article.
 * A `wbk.customArticle` message has to be send back to the basket.
 * Note: Will not be called for external catalogs.
 */
export interface HostGetCustomArticleMessage extends HostMessage
{
    type: "wbkHost.getCustomArticle";
    parameter: {
        /** The requested language for the article data in order of priority. */
        language: Array<string>;
        /** id of the articles catalog */
        catalogId: string;
        /** id of the article */
        articleId: string;
        /** Id of the currently active EAIWS tax scheme */
        taxScheme: string;
    };
}

/**
 * Will be send to request the articles of a custom catalog.
 * A `wbk.customCatalogArticles` message has to be send back to the basket.
 * Note: Will not be called for external catalogs.
 */
export interface HostGetCustomCatalogArticlesMessage extends HostMessage
{
    type: "wbkHost.getCustomCatalogArticles";
    parameter: {
        /** The requested language for the catalog data in order of priority. */
        language: Array<string>;
        /** The maxium number of results the query should return. */
        maxResults: number;
        /** Id of the catalog. If undefined articles from all available catalogs should be returned. */
        catalogId?: string;
        /** Search string for the query to perfom a full text search. */
        queryString?: string;
        /** If provided, the series id of the returned aticles should match the provided series id. */
        seriesId?: string;
        /** If provided, the base article number of the returned aticles should match the provided base article number. */
        baseArticleNumber?: string;
    };
}

/** Will be send if the project title has changed. Contains the new title as parameter. */
export interface HostProjectTitleChangedMessage extends HostMessage
{
    type: "wbkHost.projectTitleChanged";
    parameter: string;
}

/**
 * Will be send if the user clicks on the project save button. The host should then save the project.
 * The basket will be locked until the host sends the message `wbk.saveProjectDone` back to the basket.
 */
export interface HostSaveProjectMessage extends HostMessage
{
    type: "wbkHost.saveProject";
}

/**
 * Will be send as response to a `wbk.getSessionModifications` message which was send to the basket.
 * Contains a list of modifications which were made to the session as parameter.
 */
export interface HostSessionModificationsMessage extends HostMessage
{
    type: "wbkHost.sessionModifications";
    parameter: SessionModifications;
}

/**
 * Will be send every time after the user generated a report.
 */
export interface HostReportGeneratedMessage extends HostMessage
{
    type: "wbkHost.reportGenerated";
    parameter: GeneratedReport;
}

/** Will be send if finishing the integration was canceled. */
export interface HostFinishIntegrationCanceledMessage extends HostMessage
{
    type: "wbkHost.finishIntegrationCanceled";
}

/** Will be send if project editing is done. Contains a list of modifications which were made to the session as parameter. */
export interface HostDoneMessage extends HostMessage
{
    type: "wbkHost.done";
    parameter: SessionModifications;
}