import { EaiwsSession } from "@easterngraphics/wcf/modules/eaiws";
import { LoginHandler } from "./login/LoginHandler";

import { HostGetConfigurationMessage, HostGetCustomArticleMessage, HostGetCustomCatalogArticlesMessage, HostMessage } from "../libs/integration-api/HostMessages";
import { WbkMessage, WbkSaveProjectDoneMessage } from "../libs/integration-api/WbkMessages";

import { BASKET_URL, PCON_LOGIN_CLIENT_ID } from "./Config";
import { prepareAndSendConfiguration } from "./utils/BasketConfig";
import { handleCustomArticleMessage, handleCustomCatalogArticlesMessage } from "./utils/CustomCatalog";
import { saveOBK, saveOBX, savePDF } from "./utils/Export";
import { getAllItems } from "./utils/ItemData";
import { showItem } from "./utils/ItemListGui";

export class Application
{
    public get session(): EaiwsSession
    {
        return (this.mSession);
    }

    public get loginHandler(): LoginHandler
    {
        return (this.mLoginHandler);
    }

    public constructor()
    {
        this.mIFrame = document.getElementById("basketFrame") as HTMLIFrameElement; //get basket iframe
        this.mSession = new EaiwsSession();
        this.mLoginHandler = new LoginHandler(this);

        //register message handler to receive messages from basket
        window.addEventListener("message", this.onMessage);

        // setup initial value for the "Use pCon.login" checkbox
        document.addEventListener("DOMContentLoaded", () =>
        {
            (document.getElementById("checkboxUseLogin") as HTMLInputElement).checked = PCON_LOGIN_CLIENT_ID != "" && window.location.protocol.startsWith("http");
        });
    }

    public async startBasket(): Promise<void>
    {
        //login user
        const tLoggedIn = await this.login();
        if (!tLoggedIn) {
            return;
        }

        //hide intro ui
        document.getElementById("intro")!.style.display = "none";

        //load pCon.basket in integration mode and make iframe visible
        this.mIFrame.src = BASKET_URL;
        this.mIFrame.style.display = "block";
    }

    public async saveOBK(): Promise<void>
    {
        //get OBK url and download it
        window.location.href = await saveOBK(this.session);
    }

    public async saveOBX(): Promise<void>
    {
        //get url of OBX and download it
        window.location.href = await saveOBX(this.session);
    }

    public async savePDF(): Promise<void>
    {
        //get url of PDF and download it
        window.location.href = await savePDF(this.session);
    }

    public async restart(): Promise<void>
    {
        //close the session
        await this.mSession.close();

        //reload example
        window.location.reload();
    }

    /**
     * This function is used to send messages to the basket
     */
    public sendMessageToBasket(pMessage: WbkMessage): void
    {
        if (this.mIFrame.contentWindow != null) {
            this.mIFrame.contentWindow.postMessage(pMessage, "*");
        }
    }

    //// private ///////////////////////////////////////////////////////////////////////////////////

    public mIFrame: HTMLIFrameElement; //the basket iframe
    public mSession: EaiwsSession; //session object to manage EAIWS session
    public mLoginHandler: LoginHandler; //pCon.login handler

    private onMessage = (pEvent: MessageEvent): void =>
    {
        //check if message is coming from basket
        if (pEvent.source !== this.mIFrame.contentWindow)
            return;

        //call our handler
        this.handleBasketMessage(pEvent.data);
    };

    /**
     * This function handles every message coming from pCon.basket
     */
    private handleBasketMessage(pMessage: HostMessage): void
    {
        console.dir(pMessage);

        switch (pMessage.type) {
            //basket requests its configuration => prepare configuration and send it to basket
            case "wbkHost.getConfiguration": {
                const tMessage = pMessage as HostGetConfigurationMessage;
                console.log("Basket version: " + tMessage.parameter.appVersion);
                console.log("Api version: " + tMessage.parameter.messageApiVersion.major + "." + tMessage.parameter.messageApiVersion.minor);
                prepareAndSendConfiguration(this);
                break;
            }

            //basket requests the article data for a custom article
            case "wbkHost.getCustomArticle": {
                handleCustomArticleMessage(this, pMessage as HostGetCustomArticleMessage);
                break;
            }

            //basket requests the custom articles for our custom catalog
            case "wbkHost.getCustomCatalogArticles": {
                handleCustomCatalogArticlesMessage(this, pMessage as HostGetCustomCatalogArticlesMessage);
                break;
            }

            //project title changed
            case "wbkHost.projectTitleChanged": {
                //reset project title to "unnamed" if empty
                if (pMessage.parameter === "") {
                    this.sendMessageToBasket({ type: "wbk.setProjectTitle", parameter: "unnamed" });
                }
                break;
            }

            //user clicked the save project button
            case "wbkHost.saveProject": {
                this.saveProject();
                break;
            }

            //user is finished with editing
            case "wbkHost.done": {
                this.showItemList();
                break;
            }
        }
    }

    /** Login user using pCon.login */
    private async login(): Promise<boolean>
    {
        //check if login is enabled
        const tUseLogin = (document.getElementById("checkboxUseLogin") as HTMLInputElement).checked;

        //loggin with pCon.login
        if (tUseLogin) {
            const tLoggedIn = await this.mLoginHandler.loginWithPopup();
            return (tLoggedIn);
        }
        return (true);
    }

    /**
     * This function is called when the user clicks the save button in pCon.basket UI and will save an OBK file.
     */
    private async saveProject(): Promise<void>
    {
        //save OBK
        await this.saveOBK();

        //notify basket that saving is done
        let tMessage: WbkSaveProjectDoneMessage = {
            type: "wbk.saveProjectDone"
        }
        this.sendMessageToBasket(tMessage);
    }

    /**
     * This function collects all items and displays them in the item list.
     */
    private async showItemList(): Promise<void>
    {
        //close/hide basket first
        this.mIFrame.src = "";
        this.mIFrame.style.display = "none";

        //show output
        document.getElementById("output")!.style.display = "block";

        //get all items and show the
        const tItems = await getAllItems(this.mSession, true);
        for (var i = 0; i < tItems.length; ++i) {
            showItem(this.mSession, tItems[i]);
        }
    }
}