import { EaiwsSession } from "@easterngraphics/wcf/modules/eaiws";
import { AddressData, CommAddress, ContactData, FormattedText, ProjectData, ProjectText, ProjectTextType } from "@easterngraphics/wcf/modules/eaiws/project";
import { isNotNullOrEmpty } from "@easterngraphics/wcf/modules/utils/string";

/**
 * Fill header data with example project information.
 */
export function setExampleProjectData(pSession: EaiwsSession): Promise<void>
{
    const tData = new ProjectData();
    tData.projectNumber = "my_quote_00001";
    tData.projectName = "My Quote";
    tData.company = "Example Company";
    tData.customerNumber = "000012";
    tData.projectTexts = [
        createProjectText("This is my header text", "HeaderText"),
        createProjectText("This is my footer text", "FooterText")
    ];

    return pSession.project.setProjectData(tData);
}

/**
 * Fill header data with example customer information.
 */
export function setExampleCustomerAddress(pSession: EaiwsSession): Promise<void>
{
    const tCommAddr = new CommAddress();
    tCommAddr.scope = "Business";
    tCommAddr.type = "Mobile";
    tCommAddr.value = "0123/456789";

    const tContact = new ContactData();
    tContact.contactType = "Support";
    tContact.firstName = "John";
    tContact.lastName = "Doe";
    tContact.title = "Mr.";
    tContact.commAddresses = [tCommAddr];

    const tAddress = new AddressData();
    tAddress.addressType = "SoldTo";
    tAddress.name1 = "Example Company";
    tAddress.street = "Example Street 21";
    tAddress.postalCode = "123456";
    tAddress.location = "Berlin";
    tAddress.countryCode = "DE";
    tAddress.contacts = [tContact];

    return pSession.project.setAddressData(tAddress);
}

/**
 * Helper function to create an instance of `egrWcf.eaiws.project.ProjectText` with a given text as value.
 */
function createProjectText(pText: string, pType: ProjectTextType): ProjectText
{
    const tFormattedText = new FormattedText();
    tFormattedText.format = "JSON";
    tFormattedText.language = "und"; //use undefined language
    tFormattedText.value = plainTextToEaiwsJson(pText);

    const tProjectText = new ProjectText();
    tProjectText.type = pType;
    tProjectText.texts = [tFormattedText];

    return (tProjectText);
}

/**
 * Helper function to convert a plain text to the EAIWS JSON text format.
 */
function plainTextToEaiwsJson(pText: string): string
{
    const tLines = pText.trim().split("\n");
    const tParagraphContent = new Array();

    for (let i = 0; i < tLines.length; ++i) {
        let tLine = tLines[i];
        if (isNotNullOrEmpty(tLine)) {
            //EAIWS will remove consecutive spaces, replace every second space with a "non breaking space" to avoid this
            tLine = tLine.replaceAll("  ", " \u00A0");
            tParagraphContent.push(tLine);
        }
        if (i < tLines.length - 1) {
            tParagraphContent.push(["br"]);
        }
    }

    const tParagraph = ["p", null, tParagraphContent];
    const tEaiwsText = ["iml", null, [tParagraph]];
    return (JSON.stringify(tEaiwsText));
}