import { GroupedMap } from "@easterngraphics/wcf/modules/utils";

import { PluginManager } from "./PluginManager";
import { Utils } from "./Utils";
import { ComponentFactory } from "./ComponentFactory";
import { CommonResources } from "./CommonResources";
import { MessageBoxOptions } from "./MessageBox";
import { UploadDialogOptions, EaiwsSessionUploadDialogOptions } from "./UploadDialog";
import { ApplicationSettings } from "./ApplicationSettings";
import { User, UserRestriction } from "./types/User";
import { LoginDialogResult, LoginDialogOptions } from "./LoginDialog";
import { EaiwsSessionDefaults } from "./EaiwsSessionDefaults";
import { ImageLightboxOptions } from "./ImageLightbox";
import { VideoPlayerOptions } from "./VideoPlayer";
import { NoticeOptions } from "./Notice";

export interface ApiVersion
{
    /**
     * Major version will be increased for every breaking change in the api.
     * The major version required by plugins must be identical to the major api version of
     * the application, otherwise the plugin will be ignored.
     */
    major: 2;

    /**
     * Minor version will be increased for ever non-breaking change in the api.
     * The minor version required by plugins must be identical or lower to the minor api version of
     * the application, otherwise the plugin will be ignored.
     */
    minor: 5;

    /** Patch version will be increased for bug-fix releases. */
    patch: 0;

    /** If not empty the pre-release tag indicates a pre-release (e.g. alpha, beta) */
    preRelease: "";
}


export interface Application
{
    readonly name: string;

    /** Returns the current plugin api version of the application. */
    readonly apiVersion: ApiVersion;
    readonly version: string;
    readonly buildInfo: string;
    readonly buildTimeStamp: string;

    readonly settings: ApplicationSettings;

    /** eaiws session default settings */
    readonly eaiwsSessionDefaults: Readonly<EaiwsSessionDefaults>;

    readonly pluginManager: PluginManager;
    readonly utils: Utils;

    /** Factory for reusable components. */
    readonly componentFactory: ComponentFactory;

    /** Commonly used localized resources. */
    readonly commonResources: CommonResources;

    /** Returns true if the application is currently busy, see begin/endBusySection(). */
    readonly isBusy: boolean;

    /** The currently logged in user. */
    readonly user: User | null;

    /** Set of restrictions for the current user. */
    readonly userRestrictions: ReadonlySet<UserRestriction>;

    /** Can be used to store runtime user/custom data (e.g. to share data between plugins). This data is not persistent. */
    readonly customData: GroupedMap;

    /**
     * Begins a new busy section for the application. All user interactions will be blocked while
     * the application is busy. A call of beginBusySection() always has to be followed by a call of endBusySection().
     * It is allowed nest several calls to begin/endBusySection().
     */
    beginBusySection(): void;

    /** Ends the current busy section. */
    endBusySection(): void;

    /**
    * Helper function which executes the given task inside a busy section. See `beginBusySection()`.
    */
    runInBusySection<T = void>(pTask: () => Promise<T>): Promise<T>;

    /** Promise which resolves if the application is not or no longer busy. */
    waitWhileBusy(): Promise<void>;

    /**
     * Promise which resolves after the application finishes pending operations and is no longer busy.
     * Calling this function will remove the focus from the current focused element in order to call the onBlur handlers.
     * Operations which did not start a busy section will be ignored.
     */
    finishPendingOperations(): Promise<void>;

    /** Shows a message box and returns the index of the clicked button or -1 if the message box was closed by the user. */
    showMessageBox(pOptions: MessageBoxOptions): Promise<number>;

    /** Shows a notice to the user. */
    showNotice(pOptions: NoticeOptions): void;

    /** Helper function to show an error message. */
    showErrorMessage(pMessage: string, pError?: unknown): Promise<void>;

    /** Shows a common upload dialog. The promise will return after the upload is finished or the user closed the dialog. */
    showUploadDialog(pOptions: UploadDialogOptions): Promise<void>;

    /** Shows an upload dialog to upload files to an eaiws session. The urls of the uploaded files will be returned. */
    showEaiwsSessionUploadDialog(pOptions: EaiwsSessionUploadDialogOptions): Promise<Array<string>>;

    /** Shows a generic login dialog which can be used to implement user authentication. */
    showLoginDialog(pOptions: LoginDialogOptions): Promise<LoginDialogResult>;

    /** Shows a image light box (image gallery) for list of images. */
    showImageLightbox(pOptions: ImageLightboxOptions): Promise<void>;

    /** Shows/plays a video. */
    showVideo(pOptions: VideoPlayerOptions): Promise<void>;
}