import { EaiwsSession } from "@easterngraphics/wcf/modules/eaiws";
import * as project from "@easterngraphics/wcf/modules/eaiws/project";

/**
 * Interface to define a default tax rate.
 */
export interface DefaultTaxRate
{
    type: string;
    category: string;
    rate: number;
}

export type DefaultSessionSetting = "PartyInCharge" | "Currency" | "ExchangeRates" | "TaxScheme" | "DataLanguage" | "ProjectValidity";

/**
 * Default settings for eaiws sessions.
 * Can be used to assign default settings to new sessions.
 */
export interface EaiwsSessionDefaults
{
    /** default party in charge */
    readonly partyInCharge: Readonly<project.AddressData> | undefined;

    /** default currency */
    readonly currency: string | undefined;

    /** default tax scheme id */
    readonly taxScheme: string | undefined;

    /**
     * default rates for the default taxScheme
     */
    readonly taxRates: ReadonlyArray<DefaultTaxRate>;

    /**
     * default currency exchange rates (map from currency code to exchange rate)
     */
    readonly exchangeRates: ReadonlyMap<string, number>;

    /**
     * default reference currency for the exchange rates
     */
    readonly referenceCurrency: string | undefined;

    /**
     * default data language (identical to `ApplicationSettings.dataLanguage` )
     */
    readonly dataLanguage: ReadonlyArray<string>;

    /**
     * default project validity (identical to `ApplicationSettings.projectValidity` )
     */
    readonly projectValidity: string;

    /**
     * Sets new default party in charge.
     * @param pTransient If true the address will not be stored permanently and will be reset on next application start. (default: false)
    */
    setPartyInCharge(pAddress: project.AddressData | undefined, pTransient?: boolean): void;

    /** Sets new default currency. */
    setCurrency(pCurrency: string | undefined): void;

    /** Sets new default tax scheme and default rates. */
    setTaxScheme(pTaxScheme: string | undefined, pRates: Array<DefaultTaxRate> | undefined): void;

    /** Sets new default exchange rates (map from currency code to exchange rate). */
    setExchangeRates(pReferenceCurrency: string | undefined, pExchangeRates: Map<string, number>): void;

    /**
     * Applies the current defaults to the given session.
     * If `pSettingsToApply` is undefined, all settings will be applied otherwise only the settings part of the array.
     */
    applyDefaults(pSession: EaiwsSession, pSettingsToApply?: Array<DefaultSessionSetting>): Promise<void>;

    /**
     * Stores the current defaults on the user settings storage if available.
     */
    storeDefaultsOnUserStorage(): Promise<void>;
}