import { ListenableEvent } from "@easterngraphics/wcf/modules/utils";

import { Plugin } from "../Plugin";
import { PluginComponent } from "../PluginComponent";
import { MenuAction } from "../Action";

export interface ActivityHost
{
    /** Event which will be triggered if the user edited the title. */
    readonly eventTitleChanged: ListenableEvent<string, ActivityHost>;

    /** HTML container element for the activity. */
    readonly container: HTMLElement;

    /**
     * Sets/gets the current title. Setting the title does not trigger `eventTitleChanged`.
     */
    title: string;

    /** If true the user may edit the title. (default: false) */
    titleEditable: boolean;

    /**
     * Stops and disposes the activity.
     * @param pCancelable If true the stop of the activity may be canceled.
     */
    stopActivity(pCancelable: boolean): void;
}

/**
 * Additional options for activity creation.
 */
export interface ActivityOptions
{
    /**
     * If a `fileOpenActivity` was configured in the application settings, this activity will be
     * started immediately on application start if the `file_open_url` parameter is present in the application url.
     * The activity should then load the provided file immediately.
     */
    fileOpenUrl?: string;
}

/**
 * PluginComponent created when the ActivityPlugin is loaded and createComponent is called.
 * Defines the lifecycle and main menu actions for an activity.
 */
export interface Activity extends PluginComponent
{
    /** Will be called everytime the main menu gets opened. */
    getMainMenuActions(): ReadonlyArray<MenuAction> | null;

    /**
     * Can be implemented to make preparations before the activity gets stopped and disposed.
     * If `pCancelable` is true and the promise resolves with false, the activity stop will be canceled.
     * Otherwise true should be returned.
    */
    prepareActivityStop?(pCancelable: boolean): Promise<boolean>;
}

/**
 * The activity plugin is the base for all activities.
 * Activities are the entry point for the user into the application.
 * - Plugin category = `Activity`
 */
export interface ActivityPlugin extends Plugin
{
    createComponent(pHost: ActivityHost, pOptions?: ActivityOptions): Promise<Activity>;
}
