import { ListenableEvent } from "@easterngraphics/wcf/modules/utils";

import * as React from "react";
import { createRoot } from "react-dom/client";

import { PluginDescriptor } from '../../plugin-sdk/wbk-core/Plugin';
import { ActionProviderExtension, ActionProvider } from '../../plugin-sdk/wbk-core/plugins/ProjectEditorExtension';
import { ProjectEditor } from '../../plugin-sdk/wbk-core/plugins/ProjectEditor';
import { MenuAction } from '../../plugin-sdk/wbk-core/Action';

import wbkApp = egr.wbkApp;

/** implementation of ActionProviderExtension plugin interface */
class PluginImpl implements ActionProviderExtension
{
    public get name(): string
    {
        return ("examplePlugin-actionProvider");
    }

    public async initialize(pDescriptor: PluginDescriptor, pSettings: unknown): Promise<void>
    {
    }

    public async createProvider(pEditor: ProjectEditor): Promise<ActionProvider>
    {
        //return new insance of our provider implementation
        return (new Provider(pEditor));
    }
}


/** implementation of ActionProvider interface */
class Provider implements ActionProvider
{
    public eventBasketActionsChanged: ListenableEvent<void, ActionProvider>;

    public constructor(pEditor: ProjectEditor)
    {
        this.mEditor = pEditor;
        this.mSelectionAvailable = false;
        this.eventBasketActionsChanged = new ListenableEvent<void, ActionProvider>(this);

        this.mEditor.basket.eventSelectionChanged.addListener(this.onSelectionChanged);
    }

    public getBasketActions(): ReadonlyArray<MenuAction> | null
    {
        //create an action which shows the label of the first selected item
        return ([
            {
                key: "examplePlugin-exampleAction",
                text: "Example Action",
                disabled: !this.mSelectionAvailable,
                iconName: "mdi-duck",
                onExecute: () =>
                {
                    if (this.mSelectionAvailable) {
                        alert(this.mEditor.basket.selection[0].label);
                    }
                }
            }
        ]);
    }

    public getMainMenuActions(): ReadonlyArray<MenuAction> | null
    {
        //create a menu entry which opens a custom react dialog
        return ([
            {
                key: "examplePlugin-exampleMenuAction",
                text: "Example Menu Action",
                iconName: "mdi-star-face",
                onExecute: () => this.showReactDialog()
            }
        ]);
    }

    public getItemActions(): ReadonlyArray<MenuAction> | null
    {
        //create an item action which shows the label of the first selected item
        return ([
            {
                key: "examplePlugin-exampleItemAction",
                text: "Example Item Action",
                disabled: !this.mSelectionAvailable,
                iconName: "mdi-heart",
                onExecute: () =>
                {
                    if (this.mSelectionAvailable) {
                        alert(this.mEditor.basket.selection[0].label);
                    }
                }
            }
        ]);
    }

    public async dispose(): Promise<void>
    {
        //clean-up
        this.mEditor.basket.eventSelectionChanged.removeListener(this.onSelectionChanged);
    }

    //// private ///////////////////////////////////////////////////////////////////////////////////

    private mEditor: ProjectEditor;
    private mSelectionAvailable: boolean;

    private onSelectionChanged = (): void =>
    {
        const tWasAvailable = this.mSelectionAvailable;
        this.mSelectionAvailable = this.mEditor.basket.selection.length > 0;

        //trigger event to update actions
        if (tWasAvailable !== this.mSelectionAvailable) {
            this.eventBasketActionsChanged.trigger();
        }
    }

    private showReactDialog(): void
    {
        //create container for our dialog
        const tContainer = wbkApp.componentFactory.createRootComponentContainer();
        const tRoot = createRoot(tContainer.element);

        const onCloseDialog = () =>
        {
            //unmount our react component
            tRoot.unmount();

            //dispose the container
            tContainer.dispose();
        };

        //creact react elements
        let tElement =
            <div style={{
                position: "absolute",
                top: "0px",
                left: "0px",
                width: "100%",
                height: "100%",
                backgroundColor: "rgba(0,0,0,0.2)",
                zIndex: 1000
            }}>
                <div style={{
                    position: "absolute",
                    top: "calc(50% - 75px)",
                    left: "calc(50% - 100px)",
                    width: "200px",
                    height: "150px",
                    padding: "10px",
                    backgroundColor: "white",
                    border: "1px solid black"
                }}>
                    Custom Dialog
                    <br/>
                    <button onClick={onCloseDialog}>close dialog</button>
                </div>
            </div>;

        //render the react elements
        tRoot.render(tElement);
    }
}


//register plugin
wbkApp.pluginManager.registerPlugin(new PluginImpl());