import { PluginComponent } from "./PluginComponent";
import { Activity, ActivityHost } from "./plugins/Activity";
import {
    ComponentContainer,
    DataLanguagePicker,
    EaiwsRichTextEditor,
    LoadingPanel,
    ProgressOverlay,
    ProgressOverlayOptions,
    ProjectEditorActivityOptions
} from "./components";

/** Factory for reusable components. */
export interface ComponentFactory
{
    /**
     * React component which can be used to visualize a loading state.
     * If the isLoading property is true a loading animation will be shown. The user will not be able to interact
     * with the child components of the panel while loading (requires react).
     */
    loadingPanel(): LoadingPanel;

    /**
     * React component which can be used to pick a list of languages (requires react).
     */
    dataLanguagePicker(): DataLanguagePicker;

    /**
     * React rich text editor for EAIWS IML text. (requires react).
     */
    eaiwsRichTextEditor(): EaiwsRichTextEditor;

    /**
     * Creates a progress overlay which can be used to visualize a progress.
     * If the progress is no longer used, dispose() has to be called to
     * clean-up all resources.
     */
    createProgressOverlay(pOptions: ProgressOverlayOptions): ProgressOverlay;

    /**
     * Creates a new component container which can be used to display custom components at
     * application level (e.g. dialogs). If the container is no longer used, dispose() has to be called to
     * clean-up all resources.
     * @param pClassName Optional CSS class name for the container.
     */
    createRootComponentContainer(pClassName?: string): ComponentContainer;

    /**
     * Creates the default project editor activity.
     */
    createProjectEditorActivity(pHost: ActivityHost, pOptions?: ProjectEditorActivityOptions): Promise<Activity>;

    /**
     * Helper function to create a react based PluginComponent.
     */
    createReactComponent<T extends PluginComponent>(pContainer: HTMLElement, pCallback: CreateReactComponentCallback<T>): Promise<T>;
}


/**
 * Callback for `ComponentFactory.createReactComponent()`.
 * @param pSetRef Ref callback for the react component.
 * @param pDispose Has to be called if `PluginComponent.dipose()` was called.
 */
export type CreateReactComponentCallback<T extends PluginComponent> = (
    pSetRef: (pComponent: T | null) => void,
    pDispose: () => void
) => JSX.Element;