/**
 * Available plugin categories.
 * The corresponding interface for each category can be found inside the `./plugins/` folder.
 */
export type PluginCategory =
    "Authentication" |
    "Authorization" |
    "Activity" |
    "Integration" |
    "AppSettingsPanel" |

    "ProjectPicker" |
    "ProjectEditor" |
    "ProjectEditorExtension" |

    "Catalog" |
    "CatalogExtension" |
    "ProjectCalculation" |
    "ProjectCustomer" |
    "ProjectCustomerPicker" |
    "ProjectCustomerPickerExtension" |
    "ProjectData" |
    "ProjectExport" |
    "ProjectImport" |
    "ProjectOrder" |
    "ProjectPartyInCharge" |
    "ProjectReport" |
    "ProjectTax" |
    "ProjectTerms" |
    "ProjectWorkflow" |

    "ItemCalculation" |
    "ItemConfiguration" |
    "ItemConfigurationExtension" |
    "ItemImages" |
    "ItemInfo" |
    "ItemInfoExtension" |
    "ItemList" |
    "ItemListExtension" |
    "ItemProperties" |
    "ItemSelector" |
    "ItemStatusBar" |
    "ItemStatusProvider" |
    "ItemTree" |

    "AddressForm" |
    "ManufacturerDatabase" |
    "Custom";


export interface PluginDescriptor
{
    /** Name of the plugin. */
    readonly name: string;

    /** Relative path of the plugin. */
    readonly path: string;

    readonly version: string;
    readonly vendor: string;
    readonly category: PluginCategory;
    readonly subCategory: string | undefined;

    /**
     * True if this plugin is a default plugin.
     * If multiple plugins exist for the same plugin category default plugins have lower priority.
     * This is only relevant for plugin categories that support only a single implementation at runtime (e.g. `Catalog`).
     */
    readonly isDefault: boolean;

    /**
     * Optional icon defined by an image url.
     */
    readonly iconUrl: string | undefined;

    /**
     * Optional icon defined by name.
     */
    readonly iconName: string | undefined;

    /** Returns the title of the plugin using the current application language. The name is returned if the plugin does not define a title. */
    getTitle(): string;

    /** Returns the description of the plugin using the current application language. */
    getDescription(): string | undefined;

    /** Returns the absolute path to the plugin. */
    getAbsolutePath(): string;
}


export interface Plugin
{
    readonly name: string;

    /**
     * Will be called once after the plugin was loaded.
     * @param pDescriptor
     * Plugin descriptor for this plugin.
     * @param pSettings
     * Optional plugin specific settings which may be defined as part of the application settings.
     */
    initialize(pDescriptor: PluginDescriptor, pSettings: unknown): Promise<void>;
}