import { ListenableEvent } from "@easterngraphics/wcf/modules/utils";

import { MenuAction } from "../Action";
import { Plugin } from "../Plugin";
import { PluginComponent } from "../PluginComponent";
import { Disposable } from "../types/Disposable";
import { ProjectEditor } from "./ProjectEditor";

/**
 * Available sub categories for `ProjectEditorExtension` plugins
 */
export type ExtensionSubCategory =
    "ProjectPanel" |
    "ProjectDataPanel" |
    "ItemPanel" |
    "ActionProvider";


export interface BaseComponentExtension<T extends PluginComponent> extends Plugin
{
    createComponent(pEditor: ProjectEditor, pContainer: HTMLElement): Promise<T>;
}


/**
 * Extension for the project editor which shows an additionl panel/tab at project level.
 * - Plugin category = `ProjectEditorExtension`
 * - Plugin sub category = `ProjectPanel`
 */
export interface ProjectPanelExtension extends BaseComponentExtension<PluginComponent>
{
}


/**
 * Extension for the project editor which shows an additionl panel/tab at project data level.
 * - Plugin category = `ProjectEditorExtension`
 * - Plugin sub category = `ProjectDataPanel`
 */
export interface ProjectDataPanelExtension extends BaseComponentExtension<PluginComponent>
{
}


/**
 * Extension for the project editor which shows an additionl panel/tab at item level.
 * - Plugin category = `ProjectEditorExtension`
 * - Plugin sub category = `ItemPanel`
 */
export interface ItemPanelExtension extends BaseComponentExtension<PluginComponent>
{
}


/**
 * Extension for the project editor which provides additional project and item actions.
 * - Plugin category = `ProjectEditorExtension`
 * - Plugin sub category = `ActionProvider`
 */
export interface ActionProviderExtension extends Plugin
{
    createProvider(pEditor: ProjectEditor): Promise<ActionProvider>;
}


export interface ActionProvider extends Disposable
{
    /** Optional event which should be triggered if the basket actions have changed e.g. the disabled state */
    eventBasketActionsChanged?: ListenableEvent<void, ActionProvider>;

    /**
     * List of basket actions which will be available in the project editor. (e.g. in the toolbar)
     * All primary actions will be shown in front of all other actions.
     */
    getBasketActions(): ReadonlyArray<MenuAction> | null;

    /** Will be called everytime the main menu gets opened. */
    getMainMenuActions(): ReadonlyArray<MenuAction> | null;

    /** Will be called everytime the item context menu gets opened. */
    getItemActions(): ReadonlyArray<MenuAction> | null;
}

